#!/bin/sh -e
#
# Sendmail sh helper functions for Debian
#
# Copyright (c) 2001-2001, Richard Nelson <cowboy@debian.org>.
# Time-stamp: <2001/05/02 10:00:00 cowboy>
#
# Notes (to all):
#	* This is a collection of miscellaneous functions used to
#	  support /etc/init.d/sendmail, /usr/sbin/runq, etc.
#
# Notes (to self):
#   * clean_queues fubar wrt MSP
#
set -e


#------------------------------------------------------------------------------
# Parameters for the sendmail daemon
# Do *NOT* touch these lines, instead, edit /etc/mail/sendmail.conf
# The _PARMS lines are listed in precedence order
#------------------------------------------------------------------------------
Get_Parameters () {
	DAEMON_MODE="daemon"
	DAEMON_PARMS=""
	QUEUE_MODE="daemon"
	QUEUE_INTERVAL="10"
	QUEUE_PARMS=""
	MSP_MODE="cron"
	MSP_INTERVAL="$QUEUE_INTERVAL"
	MSP_PARMS="$QUEUE_PARMS"
	MISC_PARMS=""
	CRON_PARMS=""

	if [ -f /etc/mail/sendmail.conf ]; then
		. /etc/mail/sendmail.conf
		fi
	DAEMON_MODE=$(echo "$DAEMON_MODE" | tr "[:upper:]" "[:lower:]")
	QUEUE_MODE=$(echo  "$QUEUE_MODE"  | tr "[:upper:]" "[:lower:]")
	MSP_MODE=$(echo    "$MSP_MODE"    | tr "[:upper:]" "[:lower:]")

	PATH=/bin:/usr/bin:/sbin:/usr/sbin

	MTA_DAEMON=/usr/lib/sm.bin/sendmail
	MTA_COMMAND=/usr/lib/sm.bin/sendmail
	MTA_ROOT=/var/run/sendmail/mta
	MTA_PIDFILE=${MTA_ROOT}/sendmail.pid
	START_MTA_CMD="start-stop-daemon \
		--pidfile $MTA_PIDFILE \
		--exec $MTA_DAEMON \
		--startas $MTA_COMMAND \
		--start"
	STOP_MTA_CMD="start-stop-daemon \
		--pidfile $MTA_PIDFILE \
		--stop"
	SIGNAL_MTA_CMD="start-stop-daemon \
		--pidfile $MTA_PIDFILE \
		--stop"

	MSP_DAEMON=/usr/sbin/sendmail
	MSP_COMMAND=/usr/sbin/sendmail
	MSP_ROOT=/var/run/sendmail/msp
	MSP_PIDFILE=${MSP_ROOT}/sendmail.pid
	START_MSP_CMD="start-stop-daemon \
		--pidfile $MSP_PIDFILE \
		--exec $MSP_DAEMON \
		--startas $MSP_COMMAND \
		--start"
	STOP_MSP_CMD="start-stop-daemon \
		--pidfile $MSP_PIDFILE \
		--stop"
	SIGNAL_MSP_CMD="start-stop-daemon \
		--pidfile $MSP_PIDFILE \
		--stop"
	NAME=sendmail
	FLAGS="defaults 50"

	# Support for coexistance with smtpd package
	SMTPD=/usr/sbin/smtpd
	}
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# enhanced sendmail startup
#------------------------------------------------------------------------------
start_mta () {
	#
	# If already running, don't start it...
	if is_running mta; then
		echo "MTA is already running.";
		return;
		fi;
	#
	# Check if we're going to run a daemon:
	daemon_check;
	#
	# We can only afford to clean the MTA queues if running daemon mode,
	# otherwise, there is a chance that a cronjob might still be using
	# the queue...  Thats also why we don't clean the MSP queues herein.
	clean_queues;
	#
	# cd to a safe place to stash core files...
	cd $MTA_ROOT;
	#
	# if running split service, run the client queues first
	#if check_msp; then
	#	$MSP_COMMAND -q -Ac -L sm-msp-queue $MSP_PARMS $MISC_PARMS;
	#	fi;
	$START_MTA_CMD -- $MTA_PARMS &
	};
start_msp () {
	#
	# If already running, don't start it...
	if is_running msp; then
		echo "MSP is already running.";
		return;
		fi;
	#
	# Check to see if MSP mode is indeed available
	if ! check_msp; then
		return;
		fi;
	#
	# Check if we're going to run a daemon:
	if [ "$MSP_MODE" != "daemon" ]; then
		return;
		fi;
	#
	# We can only afford to clean the MSP queues if running daemon mode,
	# otherwise, there is a chance that a cronjob might still be using
	# the queue...  Thats also why we don't clean the MTA queues herein.
	#clean_queues /var/spool/mqueue-client;
	#
	# cd to a safe place to stash core files...
	cd $MSP_ROOT;
	$START_MSP_CMD -- \
		-Ac -L sm-msp -q${MSP_INTERVAL}m $MSP_PARMS $MISC_PARMS &
    };
start_sendmail () {
	if check_msp; then
		start_msp;
		fi;
	start_mta;
	};
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# enhanced sendmail shutdown
#------------------------------------------------------------------------------
stop_mta () {
	local cnt;
	stopped=0;
	#
	# If not running, don't stop it...
	if ! is_running mta; then
		return;
		fi;
	#
	# Is running, must stop it...
	$STOP_MTA_CMD --signal TERM --quiet --oknodo > /dev/null;
	#
	#       Now we have to wait until sendmail has _really_ stopped.
	#
	sleep 2;
	if $STOP_MTA_CMD --signal TERM --quiet > /dev/null; then
		echo -n "Waiting .";
		cnt=0;
		while $STOP_MTA_CMD --signal TERM --quiet > /dev/null; do
			cnt=`expr $cnt + 1`;
			if [ $cnt -gt 60 ]; then
				#
				#       Waited 120 seconds now. Fail.
				#
				echo -n " Failed ";
				stopped=1;
				break;
				fi;
			sleep 2;
			echo -n ".";
			done;
		echo -n " Done ";
		fi;
	};
stop_msp () {
	local cnt;
	stopped=0;
	#
	# If not running, don't stop it...
	if ! is_running mta; then
		return;
		fi;
	#
	# Is running, must stop it...
	$STOP_MSP_CMD --signal TERM --quiet --oknodo > /dev/null;
	#
	#       Now we have to wait until sendmail has _really_ stopped.
	#
	sleep 2;
	if $STOP_MSP_CMD --signal TERM --quiet > /dev/null; then
		echo -n "Waiting .";
		cnt=0;
		while $STOP_MSP_CMD --signal TERM --quiet > /dev/null; do
			cnt=`expr $cnt + 1`;
			if [ $cnt -gt 60 ]; then
				#
				#       Waited 120 seconds now. Fail.
				#
				echo -n " Failed ";
				stopped=1;
				break;
				fi;
			sleep 2;
			echo -n ".";
			done;
		echo -n " Done ";
		fi;
	};
stop_sendmail () {
	if check_msp; then
		stop_msp;
		fi;
	stop_mta;
	};
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# Check to see if sendmail is running
#------------------------------------------------------------------------------
is_running () {
	local result;
	result=1;
	#
	# Determine proper pidfile to check
	PIDFILE=$(echo  "$1"  | tr "[:upper:]" "[:lower:]");
	case $PIDFILE in
		mta)
			PIDFILE="$MTA_PIDFILE";
			;;
		msp)
			PIDFILE="$MSP_PIDFILE";
			;;
		*)
			PIDFILE="$1"
			;;
		esac;
	#
	# If no pidfile, not running
	# Extract pid/command and see if still running
	# Remove pidfile if app didn't
	if [ -f $PIDFILE ]; then
		PID=`head -n 1 $PIDFILE 2>/dev/null`;
		COMMAND=`tail -n 1 $PIDFILE`;
		if [ ! -z "`ps --no-heading $PID`" ]; then
			result=0;
		else
			rm -f $PIDFILE;
			fi;
		fi;
	return $result;
	};
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# Check to see if running split service (MTA, MSP)
#------------------------------------------------------------------------------
check_msp () {
	local result;
	result=1;
	if [ -s /etc/mail/submit.cf -a  -s /etc/mail/submit.mc ]; then
		if egrep -qe "^[[:space:]]*\`?FEATURE\([[:space:]]*\`?msp" \
			/etc/mail/submit.mc; then
			result=0;
			fi;
		fi;
	return $result;
	};
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# Miscellaneous sendmail command support for MSP/MTA split
# mailstats, mailq, runq
#------------------------------------------------------------------------------
mailstats () {
	#
	#if running split service, show the client status first
	if check_msp; then
		echo "MSP statistics...";
		/usr/sbin/mailstats -C /etc/mail/submit.cf $*;
		fi;
	echo "MTA statistics...";
	/usr/sbin/mailstats $*;
	echo "done.";
	};
mailq () {
	#
	# if running split service, show the client queues first
	if check_msp; then
		echo "MSP Queue status...";
		#
		# Check to see if shared memory is in use (8.12.0+)
		if egrep -qe "^[[:space:]]*\`?define\(\`?confSHAREDMEMORYKEY'?[[:space:]]*,[[:space:]]*\`?0*[1-9]+[0-9]*'?[[:space:]]*\)" /etc/mail/submit.mc; then
			$MSP_COMMAND -bP;
			fi;
		$MSP_COMMAND -bp -Ac $MISC_PARMS $*;
		fi;
	echo "MTA Queue status...";
	#
	# Check to see if shared memory is in use (8.12.0+)
	if egrep -qe "^[[:space:]]*\`?define\(\`?confSHAREDMEMORYKEY'?[[:space:]]*,[[:space:]]*\`?0*[1-9]+[0-9]*'?[[:space:]]*\)" /etc/mail/sendmail.mc; then
		$MTA_COMMAND -bP;
		fi;
	$MTA_COMMAND -bp -Am $MISC_PARMS $*;
	echo "done.";
	};
runq () {
	#
	# if running split service, run the client queues first
	if check_msp; then
		echo -n "Running the MSP queue...";
		$MSP_COMMAND -q -Ac -L sm-msp-queue $QUEUE_PARMS $MISC_PARMS $*;
		echo "done.";
		fi;
	echo -n "Running the MTA queues...";
	$MTA_COMMAND -q -Am -L sm-mta-queue $QUEUE_PARMS $MISC_PARMS $*;
	echo "done.";
	};
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# Print status of running job(s)
#------------------------------------------------------------------------------
status () {
	#
	# if running split service, check the client status first
	if check_msp; then
		if is_running msp; then
			echo "MSP: $PID $COMMAND";
		else
			echo "MSP: is not running";
			fi;
		fi;
	if is_running mta; then
		echo "MTA: $PID $COMMAND";
		/usr/share/sendmail/smcontrol.pl status;
	else
		echo "MTA: is not running";
		fi;
	};
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# Cronjob handling
#   NOTE: Should handle prior cronjob still active...
#------------------------------------------------------------------------------
cron_mta () {
	#
	# If cron not needed, don't do queue running (though it wouldn't hurt)
	if [ $QUEUE_MODE = 'cron' ]; then
		#
		# If running a split (MTA/MSP) setup, we need to make sure that
		# any messages not immediately accepted by the MTA do get delivered.
		# Only run the MSP queue if MSP_MODE=none
		#if check_msp; then
		#	if [ $MSP_MODE = 'none' ]; then
		#		#clean_queues /var/spool/mqueue-client;
		#		$MSP_COMMAND -q -Ac -L sm-msp-queue \
		#			$MSP_PARMS $MISC_PARMS $CRON_PARMS;
		#		fi;
		#	fi;

		$MTA_COMMAND -q -Am -L sm-mta-queue \
			$QUEUE_PARMS $MISC_PARMS $CRON_PARMS;
		fi;
	};
cron_msp () {
	#
	# If cron not needed, don't do queue running (though it wouldn't hurt)
	if [ $MSP_MODE = 'cron' ]; then
		#
		# If running a split (MTA/MSP) setup, we need to make sure that
		# any messages not immediately accepted by the MTA do get delivered.
		if check_msp; then
			#clean_queues /var/spool/mqueue-client;
			$MSP_COMMAND -q -Ac -L sm-msp-queue \
				$MSP_PARMS $MISC_PARMS $CRON_PARMS;
			fi;
		fi;
	};
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# Determine how to run sendmail mta daemon
#   * No daemon
#   * As listener
#   * As queue runner
#   * As both listener and queue runner
#------------------------------------------------------------------------------
daemon_check () {
	local run_daemon;
	#
	# Skip daemon run for the following:
	#   * sendmail hasn't been configured
	#   * smptd, a firewall frontend for sendmail, is installed
	#   * DAEMON_MODE = (none | inet) & QUEUE_MODE = (none | cron)
	if [ ! -s /etc/mail/sendmail.cf ] || \
	   [ ! -s /etc/mail/sendmail.mc ]; then
		echo "sendmail has not been configured, not started.";
		echo "To configure sendmail, type sendmailconfig";
		exit 1;
	elif [ -x $SMTPD ]; then
		echo "sendmail mta daemon not needed, not started.";
		exit 0;
        fi;

	MTA_PARMS="-Am -L sm-mta";
	run_daemon=3;
	case "$DAEMON_MODE" in
		none* | \
		inetd*)
			run_daemon=`expr $run_daemon - 1`;
			;;

		daemon* | \
		*)
			MTA_PARMS="$MTA_PARMS -bd $DAEMON_PARMS";
			;;
		esac

	case "$QUEUE_MODE" in
		none* | \
		cron*)
			run_daemon=`expr $run_daemon - 1`;
			;;

		daemon* | \
		*)
			MTA_PARMS="$MTA_PARMS -q${QUEUE_INTERVAL}m $QUEUE_PARMS";
			;;
		esac

	# Add any miscellanous (ie debugging) parameters
	MTA_PARMS="$MTA_PARMS $MISC_PARMS";

	if [ $run_daemon -lt 2 ]; then
		echo "sendmail listen/queue daemon not desired.";
		exit 0;
		fi;
	};
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# Clean sendmail queues (somewhat): does *not* support split qf/df/xf dirs.
# NOTE: This whole thing sucks with queue-groups, need to redo it!!!
# NOTE: Check for race conditions betwixt this code and queue-runners
#------------------------------------------------------------------------------
clean_queues () {
	local QUEUE_ROOT QUEUE QUIET
	# Obtain queue root directory
	if [ -z "$1" ]; then
		QUEUE_ROOT=/var/spool/mqueue
		QUEUE=/var/spool/mqueue
		QUIET=''
	else
		QUEUE_ROOT="$1"
		QUEUE="$1"
		QUIET='1'
		fi
	# remove lock files left because of kill/crash
	# rm -f $QUEUE/[lnx]f* doesn't work with a plethora of files ;-(
	for i in A B C D E F G H I J K L M N O P Q R S T U V W X Y Z \
		a b c d e f g h i j k l m n o p q r s t u v w x y z; do
		rm -f $QUEUE/[lnx]f${i}*
		done
	# remove zero length qf files
	#for qffile in $QUEUE/qf*; do
	for qffile in $(find $QUEUE_ROOT -type f -name qf*); do
		if [ -r "$qffile" -a ! -s "$qffile" ]; then
			if [ ! -z "$QUIET" ]; then
				echo -n "<zero: $qffile> "; fi
			rm -f "$qffile"
			fi
		done
	# rename tf files to be qf if the qf does not exist
	for tffile in $QUEUE/tf*; do
		qffile=`echo "$tffile" | sed 's/t/q/'`
		if [ -r "$tffile" -a ! -f "$qffile" ]; then
			if [ ! -z "$QUIET" ]; then
				echo -n "<recovering: $tffile> "; fi
			mv "$tffile" "$qffile"
		elif [ -f "$tffile" ]; then
			echo -n "<extra: $tffile> " 
			rm -f "$tffile"
			fi
		done
	# remove df files with no corresponding qf files
	for dffile in $QUEUE/df*; do
		qffile=`echo "$dffile" | sed 's/d/q/'`
		if [ -r "$dffile" -a ! -f "$qffile" ]; then
			if [ ! -s "$dffile" ]; then
				rm -f "$dffile"
			else
				if [ ! -z "$QUIET" ]; then
					 echo -n "<incomplete: $dffile> "; fi
				mv "$dffile" `echo $dffile | sed 's/d/D/'`
				fi
			fi
		done
	# announce files that have been saved during disaster recovery
	for xffile in $QUEUE/[A-Z]f*; do
		if [ -f "$xffile" ]; then
			if [ ! -z "$QUIET" ]; then
				echo -n "<panic: $xffile> "; fi
			fi
		done
	}
#------------------------------------------------------------------------------
